; COPYDIR.ASM program to find current directory name and MAKEDIR of the
;             same name & path another diskwith same name.
;
; Version 2:  Allows creating directory on ANY drive.  Must be given a
;             drive letter parameter. 7/26/88
;
; Version 3:  Failure to create directory returns errorcodes.  Errorcodes
;             returned are as follows:
;
;             0   Directory is there (either the root or created)
;             1   Improper drive or syntax error (no drive spec)
;             3   Path not found.  (intermediate directories don't exist)
;             5   Already exists or parent directory is full
;
; Written for CHASM assembler.  For more MASMic assemblers, ornaments will
; have to be added (like colons for the labels);
;
; Programmer: R. N. Wisan, 37 Clinton St., Oneonta, NY 13820, 8/31/87

               jmp   get_drive
orig_drive     db    0            ; drive we come from
target         db    0            ; drive to move to
target_name    db    'X:\'        ; falls thru to ASCIIZ
ASCIIZ         ds    64  ; A86 & MASM need db    64 dup 0
root_name      ds    '\',0;
cant_msg       db    'Can',39,'t make ',0
made_msg       db    'Made ',0
found_msg      db    'Found ',0
root_msg       db    'Found ROOT DIRECTORY',0
synt_msg       db    'COPYDIR needs a drive letter.',0
imp_drive_msg  db    'Improper drive.',0
err_code       db    1    ; errorlevel set on exit --defaults to 1

get_drive      mov   ah,19h       ;find current drive function
               int   21h          ;execute DOS function
               mov   orig_drive,al   ; store (0=A, 1=B)

;---Check Command Line for parameters
chk_param      mov   si,80H       ; Ptr to Command Line
               sub   bx,bx        ; zero BX & CX
               mov   cx,bx
               mov   cl,cs:[si]   ; CommandLine[0]
               cmp   cl,0         ; Any Parameter?
               je    syntax       ; No? then explain & fold

;---Read Parameter from Command Line
get_param      inc   si           ; Shift aim to next char
               mov   bl,cs:[si]   ; Get char
               and   bl,0DFh      ; upcase (drop 32-bit)
               cmp   bl,'A'       ; Alphabetic?
               jb    gp_next      ;    skip if too lo
               cmp   bl,'Z'       ;    or too hi
               ja    gp_next
               jmps  set_target   ; if you got it, ask no more
gp_next        loop  get_param    ; loop until no more comm line
               jmps  syntax       ; if still no dr let, fold

set_target     mov   target_name,bl ; store letter
               sub   bl,041h      ; conv to 0=A, 1=B &c
               mov   target,bl    ; store drive no

get_dir        mov   ah,47h       ; get dirname
               inc   al           ; change to 0=dflt, 1=A:
               mov   dl,al        ; put in dl
               mov   si,offset(ASCIIZ)
               push  cs           ; this step prob not nec.
               pop   ds           ; ds:si point to place for dirname
               int   21h          ; execute DOS function
               jmps  fix_up

fix_up         mov   bx,offset(ASCIIZ); Capitalize dir name
               call  up_case

;---Move to target drive:
ch_drive       mov   ah,0Eh       ; select drive function
               mov   dl,target    ; move to target drive
               int   21h          ; execute DOS function

               ;---find out whether we've actually moved to new drive
               mov   ah,19h       ; report current drive
               int   21h          ; (new?) curr dr -> al
               mov   dl,orig_drive
               cmp   dl,al        ;is this where we started?
               jne   root_dir     ;if not, proceed
               mov   bx,offset(imp_drive_msg) ; else fold (either
               call  write_it_out ; no such drive or user bungled.)
               jmps  home_james

syntax         mov   bx,offset(synt_msg)  ; explain & fold
               call  write_it_out
               jmps  Home_James

root_dir       mov   ah,03Bh      ; change directory
               mov   dx,offset(root_name) ; to root (of target)
               int   21h

;---Make the new directory:
make_dir       push  cs
               pop   ds
               mov   dx,offset(ASCIIZ); point to drname found
               mov   bx,dx        ;look at first char
               mov   al,[bx]      ;if zero, it's the root directory
               cmp   al,0         ;if so,
               je    root_direct  ;don't even try
               mov   ah,39h       ;make dir function
               mov   dx,offset(target_name) ; point to d:\dirname
               int   21h          ;execute DOS function
               jc    sort_out_error
               mov   bx,offset(made_msg); report
               call  write_it_out ; we made it
               sub   bh,bh        ; and set
               mov   err_code,bh  ; zero for errorlevel
               jmps  write_dir    ;if ok, skip error message

root_direct    mov   bx,offset(root_msg)
               call  write_it_out
               sub   bx,bx        ;zero for errorlevel (because this
               mov   err_code,bh  ;isn't an error -- directory is there)
               jmps  home_james

sort_out_error mov   err_code,al  ; park error number for exit
               cmp   al,4         ; if problem is in the path
               je    cant_make_it ; exit w errorlevel 3
               mov   ah,3Bh       ; move to the directory
               mov   dx,offset(target_name) ; we tried to make
               int   21h          ; if this fails, the parent is full
               jc    cant_make_it ; if error, exit w errorlevel above
               sub   bl,bl        ; if not, it was there all along
               mov   err_code,bl  ; so set errorlevel at 0
               mov   bx,offset(found_msg) ; report
               call  write_it_out ;            we found it
               jmps  write_dir    ; & exit happily

cant_make_it   mov   bx,offset(cant_msg)
               call  write_it_out ; (It's SUPPOSED to fall thru here)

write_dir      mov   bx,offset(target_name) ;(t-name falls thru to ASCIIZ)
               call  write_it_out ;write the dir name

home_james     mov   ah,0Eh       ;select drive
               mov   dl,orig_drive;where we came from
               int   21h          ;DOS function
               mov   al,err_code  ;set errorlevel
               mov   ah,04Ch      ;exit setting errorlevel
               int   21h          ;go home

write_it_out   proc  near         ;write ASCIIZ string in DS:BX
               mov   ah,2         ;display character svc
wio_next_char  mov   dl,[bx]      ;read character
               cmp   dl,0         ;is it end of string?
               jz    wio_done     ;if so stop writing
               int   21h          ;DOS function
               inc   bx           ;next char
               jmps  wio_next_char ;go get it
wio_done       ret
               endp

up_case        proc  near         ;UpCase ASCIIZ string in DS:BX
uc_start       mov   dl,[bx]      ;get char
               cmp   dl,0         ;teminator (#0)?
               jz    uc_done      ;if so skip out
               cmp   dl,'a'       ;less than "a"?
               jl    uc_next_char ;
               cmp   dl,'z'       ;greater than "z"?
               jg    uc_next_char ;
               and   dl,0DFh      ;drop 32-bit
               mov   [bx],dl      ;put it back
uc_next_char   inc   bx           ;go for next char
               jmps  uc_start
uc_done        ret
               endp
